################# Frequentist NMA Function #####################################


analyze_network_frequentist <- function(data.ab, xi, ni, 
                                        study_column_name="study", 
                                        treatment_column_name="treatment", 
                                        reference_treatment_name=NA, 
                                        rates_in_reference_treatment=c(0.01, 0.05, 0.10),
                                        digits=2) {
  
  data.ab <- as.data.frame(data.ab)
  
  ## a copy of the trt factor, to re-level
  data.ab[, "trt"] <-  data.ab[, treatment_column_name]
  
  

  es_var <- escalc(measure="PLO", xi=xi, ni=ni ) 
  data.ab[, c("yi", "vi")] <-  es_var 
  
  ## RELEVEL TRT CODE HERE 
  
  ## if there is no reference treatment name, select the lexicographically first 
  if ( is.na(reference_treatment_name)) {
    reference_treatment_name <- sort(levels(data.ab[, "trt"])[as.numeric(data.ab[, "trt"])])[1]
    cat(paste("No reference treatment given; setting reference = ", reference_treatment_name,"\n", sep=""))
  }
  
  data.ab <- within(data.ab, trt <- relevel(trt, ref=reference_treatment_name))
  
  ## count multiplicity of edges to see if we will need a random effects model  
  ## do this in igraph 
  
  gnet <- extract_graph_from_network(mtc.network(data.ab=net_data))
  edge_multiplicity <- count_multiple(gnet, eids=E(gnet))
  how_many_trts <- length(V(gnet))
  how_many_edges <- length(edge_multiplicity)
  max_comparisons_in_edges <- max(edge_multiplicity)
  
  ## The FE formula and the RE formula 
  fe_formula_string <- " ~ 1 + trt"  
  re_formula_string <- paste(" ~ 1 | ", study_column_name, "/", "trt", sep='')  
  
  if( how_many_trts >= 3 & max_comparisons_in_edges>=4 ) {
    cat(paste("running a RE analysis: max number of studies per comparison= ", max_comparisons_in_edges, "\n",sep=""))
    results <- rma.mv(yi, vi, mods= as.formula(fe_formula_string), 
                      data=data.ab, 
                      random= as.formula(re_formula_string) , 
                      method="REML")
  } else {
    cat(paste("running a FE analysis: max number of studies per comparison= ", max_comparisons_in_edges, "\n",sep=""))
    
    
    results <- try({ 
      rma.mv(yi, vi, mods= as.formula(fe_formula_string), 
             data=data.ab, 
             random= as.formula(re_formula_string) , 
             method="REML")} )
    if (is.error(results)) {
      new_fe_formula_string <- paste(" ~ 1 + trt * ", study_column_name, sep='') 
      results <- rma.uni(yi, vi, mods= as.formula(new_fe_formula_string), 
                         data=data.ab, method="FE")
      
    }
    
    
  }
  
  
  league_table <- get_league_table(network_results_with_correct_reference=results,  reference_treatment_name= paste("reference (", reference_treatment_name, ")", sep=""),
                                   treatment_column_name="trt")
  
  relative_effect_table <- get_relative_effect_table(data.ab, xi, ni, 
                                                     study_column_name=study_column_name, 
                                                     treatment_column_name=treatment_column_name,
                                                     #reference_treatment_name=NA,
                                                     digits=digits)

  ret_list <- list(
    "relative_effect_table_re" <- relative_effect_table, 
    "league_table" <- league_table)

  return(ret_list)
  
}



################# Node-Splitting Analysis Function #####################################
nodesplitting<-function(trt1,trt2,outcome,data){
    net_data<-data
    net_data$treatment <- factor(net_data$treatment)
    
    total_network<-mtc.network(net_data)
    totalretable<-get_relative_effect_table(data.ab=total_network$data.ab, 
                                            xi=total_network$data.ab$responders, 
                                            ni=total_network$data.ab$sampleSize, 
                                            study_column_name="study",
                                            # reference_treatment_name=NA, 
                                            treatment_column_name="treatment")
    #potential studies have one arm = trt1
    potential<-net_data[which(net_data$treatment==trt1),"study"]
    net_data$treatment<-as.character(net_data$treatment)
    #for each of these potential studies, see if another arm = trt2
    for (i in 1:length(potential)){
      rows<-which(net_data$study==potential[i])
      if (length(which(net_data$treatment[rows]==trt2))==1){
        #split the trt1 node in studies that have both trt1 and trt2
        net_data$treatment[which(net_data$treatment==trt1 & net_data$study==potential[i])]<-paste(trt1,"_tmp",sep="")
      }
    }
    new_network <- mtc.network(data=net_data)
    #get comparison between trt1 and trt1_tmp
    #re<-relative.effect(mtc.run(mtc.model(new_network)),trt1,paste(trt1,"_tmp",sep=""))
    cat(paste(trt1," vs. ",trt2,"\n",sep=""))
    retable<-get_relative_effect_table(data.ab=new_network$data.ab, 
                                       xi=new_network$data.ab$responders, 
                                       ni=new_network$data.ab$sampleSize,
                                       study_column_name="study",
                                       #reference_treatment_name=NA, 
                                       treatment_column_name="treatment")
    
    #pull trt1_tmp vs. trt1 from the relative effect table
    re<-retable[trt1,paste(trt1,",tmp",sep="")]
    est<-strsplit(re," ")[[1]][1]
    est<-as.numeric(gsub("[^0-9.]","",est))
    lower<-strsplit(re," ")[[1]][2]
    lower<-as.numeric(gsub("[^0-9.]", "", lower))
    upper<-strsplit(re," ")[[1]][3]
    upper<-as.numeric(gsub("[^0-9.]", "", upper))
    #pull the direct estimate from the relative effect table
    direct<-retable[paste(trt1,",tmp",sep=""),trt2]
    #pull the indirect estimate from the relative effect table
    indirect<-retable[trt1,trt2]
    #pull the network estimate from the original network relative effect table
    network<-totalretable[trt1,trt2]
    
    #calculating p-value from relative.effect output
    SE<-(upper-lower)/(2*1.96)
    Z<-est/SE
    p<-exp(-0.717*Z-0.416*(Z^2))
    
    #create a table to store values
    t<-mat.or.vec(4,3)
    t[1,1]<-paste(trt1," vs. ",trt2,sep="")
    t[2,1]<-"direct"
    t[3,1]<-"indirect"
    t[4,1]<-"network"
    t[1,2]<-p
    t[2,3]<-direct
    t[3,3]<-indirect
    t[4,3]<-network
    write.csv(t,paste("splitnode",outcome,trt1,trt2,"granular", 'csv', sep='.'))
  
}


################## League Table Function ############################
get_league_table  <- function(network_results_with_correct_reference, reference_treatment_name=NA,
                              treatment_column_name="treatment"
) {
  
  cat("in function get_league_table\n")
  
  res <- network_results_with_correct_reference
  
  treatment_names  <- gsub(paste("^", treatment_column_name, sep=''), "", rownames(res$b))
  treatment_names[1] <- reference_treatment_name
  treatment_names <- gsub("_plus_", "+", treatment_names)
  treatment_names <- gsub("_or_", "|", treatment_names)
  treatment_names <- gsub("_", ",", treatment_names)
  
  n_treatments <- length(res$b)
  
  contrasts <- diag(n_treatments-1)
  contrasts <- rbind( rep(0, n_treatments-1), contrasts )
  # league_table_list <- as.list(rep(NA, length(rates_in_reference_treatment)) ) 
  
  tmp_pred <- predict(res, newmods=contrasts, intercept=TRUE )
  
  # coerse the list.rms object to a dataframe by print() !!!
  tmp_pred <- as.data.frame(print.list.rma(tmp_pred))
  
  # turn to probabilities 
  tmp_pred <- invlogit(tmp_pred)
  
  #  Select the colums with the results. 
  #     RE models return means and predictions for a new setting 
  #     FE models return only the means 
  which_tmp_pred_columns <- is.element(colnames(tmp_pred),c("pred", "ci.lb", "ci.ub", "cr.lb", "cr.ub"))
  tmp_pred <- tmp_pred[, which_tmp_pred_columns]
  
  # name the rows and save 
  rownames(tmp_pred) <- treatment_names
  
  Estimate_Interval<-paste(format(round(100*tmp_pred$pred,1),nsmall=1)," (",
                           format(round(100*tmp_pred$ci.lb,1),nsmall=1),", ",format(round(100*tmp_pred$ci.ub,1),nsmall=1),")",sep="")
  Estimate_Interval <- gsub("( 0.0,", paste("(<0.5,", sep=""), Estimate_Interval,fixed=TRUE)
  Estimate_Interval <- gsub("^ 0.0", paste("<", 0.5, sep=""), Estimate_Interval)
  pred<-as.data.frame(Estimate_Interval,stringsAsFactors=FALSE)
  
  # IF THIS WAS A RE ANALYSIS, DO PREDICTION  
  if(is.element("cr.lb", colnames(tmp_pred))) {
    Prediction_Interval <- paste(format(round(100*tmp_pred$pred,1),nsmall=1)," (",
                                 format(round(100*tmp_pred$cr.lb,1),nsmall=1),", ",format(round(100*tmp_pred$cr.ub,1),nsmall=1),")",sep="")
    Prediction_Interval <- gsub("( 0.0,", paste("(<0.5,", sep=""), Prediction_Interval,fixed=TRUE)
    Prediction_Interval <- gsub("^ 0.0", paste("<", 0.5, sep=""), Prediction_Interval)
    pred<-as.data.frame(cbind(Estimate_Interval,Prediction_Interval),stringsAsFactors=FALSE)
  }
  
  
  rownames(pred) <- treatment_names
  
  return(pred)
  
}






################## Relative Effect Table Function ############################
get_relative_effect_table<-function(data.ab, xi, ni, 
                                    study_column_name="study", 
                                    treatment_column_name="treatment",
                                    digits=2){
  
  cat("in function get_relative_effect_table\n")
  
  data.ab <- as.data.frame(data.ab)
  
  ## a copy of the trt factor, to re-level
  data.ab[, "trt"] <-  data.ab[, treatment_column_name]
  
  es_var <- escalc(measure="PLO", xi=xi, ni=ni ) 
  data.ab[, c("yi", "vi")] <-  es_var 
  
  ## set up the empty table 
  relative_effect_table <- matrix(,nrow=length(levels(data.ab$treatment)),
                                  ncol=length(levels(data.ab$treatment)))
  relative_effect_table <- as.data.frame(relative_effect_table)
  
  reference_treatment_name_list <- unique(levels(data.ab[,"trt"])[as.numeric(data.ab[,"trt"])])
  reference_treatment_name_list <- reference_treatment_name_list[order(reference_treatment_name_list)]
  
  
  ## count multiplicity of edges to see if we will need a random effects model  
  ## do this in igraph 
  
  gnet <- extract_graph_from_network(mtc.network(data.ab=net_data))
  edge_multiplicity <- count_multiple(gnet, eids=E(gnet))
  how_many_trts <- length(V(gnet))
  how_many_edges <- length(edge_multiplicity)
  max_comparisons_in_edges <- max(edge_multiplicity)
  
  ## The FE formula and the RE formula 
  fe_formula_string <- " ~ 1 + trt"  
  re_formula_string <- paste(" ~ 1 | ", study_column_name, "/", "trt", sep='')  
  
  run_FE <- FALSE 
  run_RE <- FALSE 
  
  if( how_many_trts >= 3 & max_comparisons_in_edges>=4 ) {
    cat(paste("running a RE analysis: max number of studies per comparison= ", max_comparisons_in_edges, "\n",sep=""))
    run_RE <- TRUE
  } else {
    cat(paste("running a FE analysis: max number of studies per comparison= ", max_comparisons_in_edges, "\n",sep=""))
    run_FE <- TRUE   # superfluous 
  }
  
  
  
  y<-0
  for (ref in reference_treatment_name_list) {
    y<-y+1
    
    data.ab <- within(data.ab, trt <- relevel(trt, ref=ref))
    
    if(run_RE==TRUE) {
      results <- rma.mv(yi, vi, mods= as.formula(fe_formula_string), 
                        data=data.ab, 
                        random= as.formula(re_formula_string) , 
                        method="REML")
    } else {
      
      
      
      results <- try({ 
        rma.mv(yi, vi, mods= as.formula(fe_formula_string), 
               data=data.ab, 
               random= as.formula(re_formula_string) , 
               method="REML")} )
      if (is.error(results)) {
        new_fe_formula_string <- paste(" ~ 1 + trt * ", study_column_name, sep='') 
        results <- rma.uni(yi, vi, mods= as.formula(new_fe_formula_string), 
                           data=data.ab, method="FE")}
      
      
      
    }
    
    
    # exponentiate before packing results, to get odds ratios,
    est <- exp(results$b)
    low <- exp(results$ci.lb)
    high <- exp(results$ci.ub)
    
    est_str <- paste(round(est, digits=digits))
    est_str <- gsub("^0$", paste("<", 0.5*10^(-digits), sep=""), est_str)
    
    low_str <- paste(round(low, digits=digits))
    low_str <- gsub("^0$", paste("<", 0.5*10^(-digits), sep=""), low_str)
    
    high_str <- paste(round(high, digits=digits))
    high_str <- gsub("^0$", paste("<", 0.5*10^(-digits), sep=""), high_str)
    
    relative_effect_ref <- paste(est_str, " (", 
                                 low_str, ", ",
                                 high_str, ")", sep="")
    
    relative_effect_ref <- as.data.frame(relative_effect_ref, stringsAsFactors=FALSE)
    
    treatment_names  <- rownames(results$b)
    treatment_names  <- gsub("trt","", treatment_names)
    treatment_names[1] <- ref
    
    treatment_names <- gsub("_plus_", "+", treatment_names)
    treatment_names <- gsub("_or_", "|", treatment_names)
    treatment_names <- gsub("_", ",", treatment_names)
    
    
    rownames(relative_effect_ref) <- treatment_names
    relative_effect_ref[1, 1] <- ref
    relative_effect_table[, y] <- relative_effect_ref[order(rownames(relative_effect_ref)),]
    
  }
  
  colnames(relative_effect_table) <- treatment_names[order(treatment_names)]
  rownames(relative_effect_table) <- treatment_names[order(treatment_names)]
  return(relative_effect_table)
}

############################# Extracting Graph from Network Object ######################
extract_graph_from_network <- function(network) {
  
  data <- network$data.ab 
  #node_names <- levels(data$treatment)  
  node_names <- unique(levels(data$treatment)[as.numeric(data$treatment)])
  
  # make an empty graph 
  g0 <- make_empty_graph(length(node_names), directed=FALSE) 
  
  g <- set_vertex_attr(g0, "name", V(g0), node_names)
  
  ### add total number of people per node  
  for (node in node_names) {
    g <- set_vertex_attr(g, "sampleSize", V(g)[node], sum(data[is.element(data$treatment, node), "sampleSize"]) )
  }
  
  study_list <- unique(data$study)
  
  for (st in study_list) {
    trt <- unique(data[data$study==st, "treatment"])
    
    for (i in 1:(length(trt)-1) ) {
      for (j in (i+1):length(trt))  {
        g <- add_edges(g, c(which(V(g)$name==trt[i]), which(V(g)$name==trt[j])))	
        
        which_edge <- length(E(g))
        g <- set_edge_attr(g, "num_studies", which_edge, 1 )
        
        which_rows <- (data$study == st) & ((data$treatment == trt[i]) | (data$treatment == trt[j]))
        g <- set_edge_attr(g, "sampleSize", which_edge, sum(data[which_rows, "sampleSize" ]))
        
        v_1 <- get.edges(g, E(g)[which_edge])[1]
        g <- set_edge_attr(g, "node_name_1", which_edge, vertex_attr(g, "name", v_1))
        
        which_row_node_1 <- (data$study == st) & (data$treatment == trt[i]) 
        g <- set_edge_attr(g, "node_1", which_edge, trt[i])    
        g <- set_edge_attr(g, "x_node_1", which_edge,  data[which_row_node_1, "responders" ])   				
        g <- set_edge_attr(g, "n_node_1", which_edge,  data[which_row_node_1, "sampleSize" ])  
        g <- set_edge_attr(g, "author_node_1", which_edge, data[which_row_node_1, "author" ])
        g <- set_edge_attr(g, "location_node_1", which_edge, data[which_row_node_1, "location" ])
        g <- set_edge_attr(g, "arm_node_1", which_edge, data[which_row_node_1, "arm" ])
        g <- set_edge_attr(g, "RoB", which_edge, data[which_row_node_1, "Risk of Bias" ])
        
        
        v_2 <- get.edges(g, E(g)[which_edge])[2]
        g <- set_edge_attr(g, "node_name_2", which_edge, vertex_attr(g, "name", v_2))
        which_row_node_2 <- (data$study == st) & (data$treatment == trt[j]) 
        g <- set_edge_attr(g, "node_2", which_edge, trt[j])    
        g <- set_edge_attr(g, "x_node_2", which_edge,  data[which_row_node_2, "responders" ])   				
        g <- set_edge_attr(g, "n_node_2", which_edge,  data[which_row_node_2, "sampleSize" ])   
        g <- set_edge_attr(g, "author_node_2", which_edge, data[which_row_node_2, "author" ])
        g <- set_edge_attr(g, "location_node_2", which_edge, data[which_row_node_2, "location" ])
        g <- set_edge_attr(g, "arm_node_2", which_edge, data[which_row_node_2, "arm" ])
        g <- set_edge_attr(g, "time_str", which_edge, data[which_row_node_2, "time" ])
        #g <- set_edge_attr(g, "time_units", which_edge, data[which_row_node_2, "time_units" ])
      }
    }			
  }
  
  return(g)
  
}

logit <- function(p) {
  return( log(p) - log(1-p) )
}

invlogit <- function (l) {
  return (exp(l)/(1+exp(l)) )
}


#I had to implement a try() for when the RE cannot be run  
# In that case one would have to 

is.error <- function(x) {
  inherits(x, "try-error")
}